#! /usr/local/bin/perl -w

# 1315, Thu 6 Jan 00
#
# Nevil Brownlee, ITSS Technology Development, The University of Auckland

# Takes a *.plt file, looks at cols 2 .., adds blank lines if any column has
#    more than $brk_len consecutive missing points (Brad's algorithm)
#          
# ./g_brk.pl  from_file
#       e.g. ua.dns-week.000617.plt 
#
# The from_file is saved as from_file~, and a new version of from_file created

use strict;
use Time::Local;

local %::month = (
      "Jan" =>  1, "Feb" =>  2, "Mar" =>  3, "Apr" =>  4,
      "May" =>  5, "Jun" =>  6, "Jul" =>  7, "Aug" =>  8,
      "Sep" =>  9, "Oct" => 10, "Nov" => 11, "Dec" => 12,
   );

my $brk_len = 3;  # Max nbr of missing consecutive points

my ($from_file, $line, $to_file);

$from_file = shift;  # First item from command line
if (defined($from_file)) {
   open(FROM, $from_file)
   or die "Can't open from_file: $!\n";
   }
else { printf "No from_file specified!\n"; }

close(FROM);
rename($from_file, "$from_file~~");
open(FROM, "$from_file~~");
open(TO, ">$from_file");

my $nbr = 0;
my @last_row = ();
my @seen = ();
my $first_sample = 1;
my $blank_line = 0;

my @run_length = ();

while (<FROM>) { # read  flows file

   if (/^\#/) { # starts with a #
      print TO $_;  # Copy # records to output file
#      if (s/^##(plt_stat|loss_filter)\.pl //) {
      if (s/^##loss_filter //) {
         print TO "##g_brk  brk_len=$brk_len\n";
	 } 
      }
   else {
      if (length($_) == 1) {  # Blank line
         print TO $_;
         $blank_line = 1;
         next;
        }

      chop;
      my @row = split(/ +/, $_);  # Split whole input record into its values
      my $x = shift(@row);

      if ($first_sample) {
         for (my $j = 0; $j != scalar(@row); ++$j) { $run_length[$j] = 0; }
         }

      if (!$first_sample && !$blank_line) {
         my $mx_run = 0;
         for (my $j = 0; $j != scalar(@row); ++$j) {
            if (($row[$j] eq '?' && $last_row[$j] ne '?') ||
                  ($row[$j] ne '?' && $last_row[$j] eq '?'))
               { $run_length[$j] = 0; }
            elsif ($row[$j] eq '?')
	      { ++$run_length[$j]; }
            if ($run_length[$j] > $mx_run) { $mx_run = $run_length[$j]; }
            }
         if ($mx_run > $brk_len) { 
            print TO "\n"; 
            $blank_line = 1;
            }
         }
      else { $first_sample = 0; }
      @last_row = @row;

      if ($blank_line) { 
         for (my $j = 0; $j != scalar(@row); ++$j) { $run_length[$j] = 0; }
         $blank_line = 0;
         }

#      print TO "$x ";
      print TO "$_\n";
      }


#   ++ $nbr;
#   last if $nbr == 6;
   }

