#! /usr/local/bin/perl -w

# plot_data-rate.pl: called by plot_dr to do data-rate plots
#
# Nevil Brownlee, CAIDA / University of Auckland
#
# 1015, Sat 14 Oct 2000

#  ./plot_dr.pl -d meter -m sdsc -r data-rate -s 20000603 -n 7 -i dr-week.000617.dif -v


$^W = 1;  # Warnings on
use strict;
#use Symbol;
use Time::Local;


my $home = "/home/nevil";

my %pc = (
   "dark"    =>  3,  # blue (was grey)
   "medium"  =>  1,  # red
   "light"  =>  5   # magenta (was green)
   );

my ($l1_c, $l2_c, $d1_c, $d2_c);

my ($base_dir, $out_prefix,
   $data_in, $place_time, $site, $tz,
   $start_day, $n_days, $anon, $logo_reqd, $bl_wh, $eps_plot)
   = @ARGV;  # Args from command line

chdir $base_dir || die "Can't chdir to $base_dir: $!";

my $xstart = g_fmt_date($start_day);
my $fin_day = modify_date($start_day, $n_days);
my $xfinish = g_fmt_date($fin_day);

print "base_dir=$base_dir, out_prefix=$out_prefix,\n" .
   " data_in=$data_in, place_time=$place_time,\n" .
   " site=$site, tz=$tz,\n" .
   " start_day=$start_day, n_days=$n_days,\n" .
   " anon=$anon, logo_reqg=$logo_reqd, bl_wh=$bl_wh, eps_plot=$eps_plot\n";

my ($nstats,$ndistribs,$ncols, $utc_in,$time_label);
get_dist_params($data_in, \$nstats,\$ndistribs,\$ncols);
my $n_servers = $ncols/2;
print "nstats=$nstats, ndistribs=$ndistribs, ncols=$ncols, n_servers=$n_servers\n";
if ($utc_in) {
   $time_label = 'Time: ddThh (UTC)';
   }
else {
   $time_label = "Local Time$tz";
   }
if ($bl_wh) {  # Only use black and grey
   $l1_c = 0;   # grey
   $d1_c = -1;  # black
   $l2_c = 0;   # grey
   $d2_c = -1;  # black
   }
else {  # Use colours
   $l1_c = 8;  # green, 50%
   $d1_c = 3;  # blue, 75%
   $l2_c = 5;  # magenta, 95%
   $d2_c = 1;  # red
   }

# Set globals for plot routines

my $logo_x = g_fmt_date($start_day, $n_days == 1 ? +30 : +3*60);
my $ylab_x = g_fmt_date($start_day, $n_days == 1 ? -10 : -2*60);
my $right  = g_fmt_date($fin_day, $n_days == 1 ? -30 : -3*60);
print "fin_day=$fin_day, logo_x=$logo_x, ylab_x=$ylab_x, right=$right\n";

my ($tempname, $plotname);
$tempname = "$out_prefix.tmp";
if (!open(PLOT, ">$tempname") ) {
   printf STDERR "Couldn't open $tempname for write <<<\n";
   exit;
    }

my $max_dr = ($site eq 'SDSC') ? 23 : 49.5;  # Max y value
my $units_y = ($site eq 'SDSC') ? 21.5 : 47.5;
my $plotname_y = ($site eq 'SDSC') ? 21 : 47;
my $logo_y = ($site eq 'SDSC') ? -4.2 : -8.5;

   $plotname = 'data-rate';
   put_gnu_header(\*PLOT, 't', $xstart,$xfinish);
   put_multiplot(\*PLOT, $data_in, $site, $tz,
   "10-second Data Rate distributions $place_time",
   0,$max_dr, $units_y,"Mbps", $plotname_y,$logo_y,
      "maximum", 6, "minimum", 4, "median", 2, '', 0);
   close (PLOT);

   if ($eps_plot) {
      printf ">>> gnuplot $tempname > $out_prefix.$plotname.eps\n";
      system("gnuplot $tempname > $out_prefix.$plotname.eps");
    }
   else {
      printf ">>> gnuplot $tempname > $out_prefix.$plotname.png\n";
      system("gnuplot $tempname > $out_prefix.$plotname.png");
      }
   #   unlink "$tempname";

# ----------------------------------------------------------

sub get_dist_params {
   my ($from_file, $st_ref,$dist_ref,$cols_ref) = @_;  # Arguments

   if (defined($from_file)) {
      open(FROM, $from_file)
      or die "Can't open from_file: $!\n";
      }
 
   my (@attribs, @lo, @hi, @nvalues);
   my $n = 0;  my $mycols = 1;

   while (<FROM>) { # read  flows file
      chop;  # Trim new line
      if (/^#/) {  # Starts with a #
         if (/^##NeTraMet /) {
            if (/\(UTC\)/) {  $utc_in = 1; }
	     print " ==== utc_in = 1\n";
            }
         elsif (/^##plt_stat /) {
            my @items = split(/ +/, $_);
            foreach my $item (@items) {
               if ($item =~ /(.+)=(\d+)\.\.(\d+)/) {
                  $attribs[$n] = $1;
	          $lo[$n] = $2;  $hi[$n] = $3;
                  $nvalues[$n] = $3-$2+1;
	          $mycols *= $nvalues[$n];
	          ++$n;
                  }
               elsif ($item =~ /stats=(\d+)/) {
                  $$st_ref = $1;
                  }
               elsif ($item =~ /distribs=(\d+)/) {
                  $$dist_ref = $1;  $mycols *= $ndistribs;
                  }
               elsif ($item =~ /cols=(\d+)/) {
                  $$cols_ref = $1;
                  }
               }
            close(FROM);
            return;
	    }
	 } 
      next;
      }
   }


sub put_multiplot {
   my ($FILE, $data_in, $site, $tz, $p_title,
      $ylo,$yhi,$ylab_y,$yunits,  $ykey,$ytag,
      $key1,$c1, $key2,$c2, $key3,$c3, $key4,$c4) = @_;  # Arguments

   print $FILE "set multiplot\n";
#   print $FILE "set logscale y\n";
   print $FILE "set key bottom right\n";
   print $FILE "set title \"$p_title\"\n";

   print $FILE "set origin 0.01, 0.46\n";
   print $FILE "set size 0.97, 0.50\n";
#   print $FILE "set origin 0.01, 0.48\n";
#   print $FILE "set size 0.97, 0.49\n";
   print $FILE "set yrange [$ylo:$yhi]\n";
   print $FILE "set label 1 \"$yunits\" at $ylab_x, $ylab_y right\n";
   print $FILE "set label 2 \"Traffic out from $site [ToBitrate]\" at $right,$ykey right\n";

   print $FILE "plot \"$data_in\" u 1:$c1 t \"$key1\" w li $l2_c" .  # Max
                  ", \"$data_in\" u 1:$c3 t \"$key3\" w li $d1_c" .  # Median
                  ", \"$data_in\" u 1:$c2 t \"$key2\" w li $l1_c";   # Min
   if ($key4 ne '') 
      { print $FILE ", \"$data_in\" u 1:$c4 t \"$key4\" w li $d2_c"; }
   print $FILE "\n";

   print $FILE "set title\n";
#   print $FILE "set nokey\n";
   print $FILE "set origin 0.01, 0\n";
   print $FILE "set size 0.97, 0.48\n";
#   print $FILE "set size 0.97, 0.43\n";
#   print $FILE "set origin 0.01, 0.05\n";

   print $FILE "set label 2 \"Traffic in to $site [FromBitRate]\" at $right,$ykey right\n";
   print $FILE "set xlabel \"$time_label\" +32, +0\n";
#   print $FILE "set xlabel \"Local Time, $tz\" +25, +0\n";
   if ($logo_reqd) {
      print $FILE
         "set label 3 \"Coral NeTraMet, 5-min samples\\n" .
         "Nevil Brownlee, U Auckland / CAIDA\" at $logo_x, $ytag\n";
      }
   ++$c1;  ++$c2;  ++$c3;  ++$c4;
   print $FILE "plot \"$data_in\" u 1:$c1 t \"$key1\" w li $l2_c" .  # Max
                  ", \"$data_in\" u 1:$c3 t \"$key3\" w li $d1_c" .  # Median
                  ", \"$data_in\" u 1:$c2 t \"$key2\" w li $l1_c";   # Min
   if ($key4 ne '') 
      { print $FILE ", \"$data_in\" u 1:$c4 t \"$key4\" w li $d2_c"; }
   print $FILE "\n";

   print $FILE "set nomultiplot\n";
   }

sub put_gnu_header {
   my ($FILE, $timefmt, $xstart,$xfinish) = @_;  # Arguments

   if ($eps_plot) {
      print $FILE "set term postscript eps monochrome\n";
#      print $FILE "set term postscript eps color\n";
      }
   else {
      print $FILE "set term png small color\n";
      }

#  print $FILE "set term gif small size 640,480 xffffff x000000 x404040 \\\n",
#                 "xff0080 xff6600 x00ff40 x0080ff\n";  # red -> blue
# last three colours used for 10, 3, 1 contours
# if gnuplot finds four contours, it uses four colours, so we'd need
#    to specify one more!
#                 "xff0080 xff0000 xff6600 xfca503 \\\n",
#                 "xffffd8 x00ff40 x00ffff x0080ff\n";  # red -> blue

   print $FILE "set missing \"?\"\n";
   if ($timefmt ne '') {
      print $FILE "set timefmt \"%m%d.%H%M\"\n";
      print $FILE "set xdata time\n";
      print $FILE "set format x \"%02dT%H\"\n";  # ISO 8601 uses T (Ken Keys)
      if (defined($xstart)) {
         print PLOT "set xrange [$xstart:$xfinish]\n";
         }
      }
   }


sub u_seconds {
   my ($t_day) = @_;  # Arguments

   my ($fy, $fm, $fd);
   $t_day =~ /^(\d\d\d\d)(\d\d)(\d\d)/;
   $fy = $1;  $fm = $2;  $fd = $3;
   return timelocal(0, 0, 0, $fd, $fm-1, $fy-1900);
   }

sub modify_date {
   my ($t_day, $d_incr) = @_;  # Arguments

   my $fs = u_seconds($t_day);

   my @fyd = localtime($fs + $d_incr * 24*60*60);  # + hours
   my $r = sprintf "%04d%02d%02d",
      $fyd[5]+1900, $fyd[4]+1, $fyd[3];  # yyyy mm dd
   return $r;
   }

sub g_fmt_date {
   my ($t_day, $h_incr) = @_;  # Arguments

   my $fs = u_seconds($t_day);

   if (defined($h_incr)) { $fs += $h_incr*60; }

   my @fyd = localtime($fs);
   my $r = sprintf '"%02d%02d.%02d%02d"',
      $fyd[4]+1, $fyd[3], $fyd[2], $fyd[1];  # mm dd hh mm
   return $r;
   }
