#! /usr/local/bin/perl -w

# plot_flow-stats-size.pl: called by plot_dr to do stream octet, time plots
#
# Nevil Brownlee, CAIDA / University of Auckland
#
# 1015, Sat 14 Oct 2000

#  ./plot_dr.pl -d meter -m sdsc -r flow-stats-size -s 20000617 -n 7 -p -v
#  ./plot_dr.pl -d meter -m sdsc -r flow-stats-size -s 20001029 -n 2 -p -v
#  ./plot_dr.pl -d d/meter -m ucsd -r flow-stats-size -s 20010202 -n 5 -p -v

$^W = 1;  # Warnings on
use strict;
#use Symbol;
use Time::Local;

my $home = "/home/nevil";

my ($base_dir, $out_prefix,
   $data_in, $place_time, $site, $tz,
   $start_day, $n_days, $anon, $logo_reqd, $bl_wh, $eps_plot)
   = @ARGV;  # Args from command line

chdir $base_dir || die "Can't chdir to $base_dir: $!";

my $xstart = g_fmt_date($start_day);
my $fin_day = modify_date($start_day, $n_days);
my $xfinish = g_fmt_date($fin_day);

print "plt_flow-stats-size ..\n" . 
   " base_dir=$base_dir, out_prefix=$out_prefix,\n" .
   " data_in=$data_in, place_time=$place_time,\n" .
   " site=$site, tz=$tz,\n" .
   " start_day=$start_day, n_days=$n_days,\n" .
   " anon=$anon, logo_reqg=$logo_reqd, bl_wh=$bl_wh, eps_plot=$eps_plot\n";

my ($l1_c, $l2_c, $d1_c, $d2_c);
if ($bl_wh) {  # Just use dark_c for strip charts
   $l1_c = 0;   # grey
   $d1_c = -1;  # black
   $l2_c = 0;   # grey
   $d2_c = -1;  # black
   }
else {  # Use colours
   $l1_c = 8;  # green, 50%
   $d1_c = 3;  # blue, 75%
   $l2_c = 5;  # magenta, 95%
   $d2_c = 1;  # red
   }

# data_in is a prefix here -
#    actual data filenames are formed by adding
#
#1: lifetimes  (1q,2q,3q) from $iprefix.flowtime.tcp/udp
#2: frompdus   (1q,2q,3q) from $iprefix.frompdus.t/u
#3: topdus     (1q,2q,3q) from $iprefix.topdus.t/u
#4: fromoctets (1q,2q,3q) from $iprefix.fromoctets.t/u
#5: tooctets   (1q,2q,3q) from $iprefix.tooctets.t/u

# Set globals for plot routines

my $logo_x = g_fmt_date($start_day, $n_days == 1 ? +30 : +3*60);
my $ylab_x = g_fmt_date($start_day, $n_days == 1 ? -10 : -2*60);
my $right  = g_fmt_date($fin_day, $n_days == 1 ? -30 : -3*60);
print "fin_day=$fin_day, logo_x=$logo_x, ylab_x=$ylab_x, right=$right\n";

my ($nstats,$ndistribs,$ncols, $utc_in,$time_label);


plot_streams($out_prefix, 1, $data_in, $site, $tz,
   "Stream Lifetime [FlowTime] $place_time");
plot_streams($out_prefix, 4, $data_in, $site, $tz,
   "Inbound stream size [FromFlowOctets] $place_time");
plot_streams($out_prefix, 5, $data_in, $site, $tz,
   "Outbound stream size [ToFlowOctets] $place_time");

plot_streams($out_prefix, 16, '0202.2200', $site, $tz,
   "Cumulative stream lifetime distribution [FlowTime] at $site (%)");
plot_streams($out_prefix, 17, '0202.2200', $site, $tz,
   "Cumulative stream size distribution [FromFlowOctets] at $site (%)");
plot_streams($out_prefix, 18, '0202.2200', $site, $tz,
   "Cumulative stream size distribution [ToFlowOctets] at $site (%)");
exit;


sub plot_streams {
   my ($out_prefix, $which, $iprefix, 
      $site, $tz, $title) = @_;  # Arguments

   my ($tempname, $plotname);
   $out_prefix =~ /^([^\/]+)/;
   my $ruleset = $1; 

   if ($which == 1) {
      $plotname = 'flowtime';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot(\*PLOT, $iprefix,
         $site, $tz, $title,
         12, 4, 1,
         "max", "95 %", "75 %", "50 %",
         4,"UDP", 7,"non-WWW", 10,"outside WWW", 13,"inside WWW");
      close (PLOT);
      }
   elsif ($which == 4) {  # FromFlowOctets
      $plotname = 'fromflowoctets';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot(\*PLOT, $iprefix,
         $site, $tz, $title,
         12, 4, 2,
         "max", "95 %", "75 %", "50 %",
         3,"UDP", 6,"non-WWW", 9,"outside WWW", 12,"inside WWW");
      close (PLOT);
      }
   elsif ($which == 5) {  # ToFlowOctets
      $plotname = 'toflowoctets';
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot(\*PLOT, $iprefix,
         $site, $tz, $title,
         12, 4, 2,
         "max", "95 %", "75 %", "50 %",
         2,"UDP", 5,"non-WWW", 8,"outside WWW", 11,"inside WWW");
      close (PLOT);
      }

   elsif ($which == 2) {
      $plotname = "$iprefix.frompdus";
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot(\*PLOT, $site, $tz, $title,
         "Stream size (packets) in to SDSC [FromFlowPDUs]",
         0.8,80000, 400,0.1, "pkt", 30000,
         "maximum", 6, "median", 2, "3rd quartile", 4, "95th percentile", 8);
      close (PLOT);
      }
   elsif ($which == 3) {
      $plotname = "$iprefix.topdus";
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_multiplot(\*PLOT, $site, $tz, $title,
         "Stream size (packets) out from SDSC [ToFlowPDUs]",
         0.8,80000, 400,0.1, "pkt", 30000,
         "maximum", 6, "median", 2, "3rd quartile", 4, "95th percentile", 8);
      close (PLOT);
      }

   elsif ($which == 6) {
      $plotname = "flowtime.$iprefix.dist";
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_distrib_plot(\*PLOT, $site, $tz, $title,
         $iprefix, $plotname, 1,
         3,"UDP", 4,"non-WWW", 5,"outside WWW", 6,"inside WWW");
      close (PLOT);
      }
   elsif ($which == 16) {
      $plotname = "flowtime.$iprefix.cdst";
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_distrib_plot(\*PLOT, $site, $tz, $title,
         $iprefix, $plotname, 2,
         3,"UDP", 4,"non-WWW", 5,"outside WWW", 6,"inside WWW");
      close (PLOT);
      }
   elsif ($which == 17) {
      $plotname = "fromflowoctets.$iprefix.cdst";
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_distrib_plot(\*PLOT, $site, $tz, $title,
         $iprefix, $plotname, 3,
         3,"UDP", 4,"non-WWW", 5,"outside WWW", 6,"inside WWW");
      close (PLOT);
      }
   elsif ($which == 18) {
      $plotname = "toflowoctets.$iprefix.cdst";
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      put_distrib_plot(\*PLOT, $site, $tz, $title,
         $iprefix, $plotname, 3,
         3,"UDP", 4,"non-WWW", 5,"outside WWW", 6,"inside WWW");
      close (PLOT);
      }

   elsif ($which == 7) {
      $plotname = "$iprefix.flowpdus.dist";
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      print PLOT "set title \"Stream Size distribution observed at SDSC, " .
         "00.$iprefix\"\n";
      print PLOT "set xlabel \"Stream Size, packets\" +25, +0\n";
      print PLOT "set logscale xy\n";
      print PLOT "set xrange[0.8:65000]\n";
      print PLOT "set yrange[0.8:30000]\n";
      print PLOT "set label \"Coral/DAG NeTraMet, 5-min samples\\n" .
                  "Nevil Brownlee, U Auckland / CAIDA\" at 55000,3000 right\n";
      print PLOT "plot \"$plotname.plt\"" .
               " u 2:3 t \"TCP out of SDSC [ToFlowPDUs]\" w st 5" . # magenta
         ", \"\" u 2:4 t \"TCP into SDSC [FromFlowPDUs]\" w st 1" . # red
         ", \"\" u 2:5 t \"UDP out of SDSC [ToFlowPDUs]\" w st 4" . # cyan
         ", \"\" u 2:6 t \"UDP into SDSC [FromFlowPDUs]\" w st 3" . # blue
         "\n";
      close (PLOT);
      }
   elsif ($which == 8) {
      $plotname = "$iprefix.flowoctets.dist";
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      print PLOT "set title \"Stream Size distribution observed at SDSC, " .
         "00.$iprefix\"\n";
      print PLOT "set xlabel \"Stream Size, kiloBytes\" +25, +0\n";
      print PLOT "set logscale xy\n";
      print PLOT "set xrange[0.08:6500]\n";
      print PLOT "set yrange[0.8:60000]\n";
      print PLOT "set label \"Coral/DAG NeTraMet, 5-min samples\\n" .
                  "Nevil Brownlee, U Auckland / CAIDA\" at 5000,2000 right\n";
      print PLOT "plot \"$plotname.plt\"" .
               " u 2:3 t \"TCP out of SDSC [ToFlowOctets]\" w st 5" . # magenta
         ", \"\" u 2:4 t \"TCP into SDSC [FromFlowOctets]\" w st 1" . # red
         ", \"\" u 2:5 t \"UDP out of SDSC [ToFlowOctets]\" w st 4" . # cyan
         ", \"\" u 2:6 t \"UDP into SDSC [FromFlowOctets]\" w st 3" . # blue
         "\n";
      close (PLOT);
      }
   elsif ($which == 17) {
      $plotname = "$iprefix.flowpdus.cdst";
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      print PLOT "set title \"Cumulative Stream Size distribution observed at SDSC, " .
         "00.$iprefix\"\n";
      print PLOT "set xlabel \"Stream Size, packets\" +25, +0\n";
      print PLOT "set logscale x\n";
      print PLOT "set xrange[0.8:65000]\n";
      print PLOT "set yrange[0:100]\n";
      print PLOT "set ytics 0,10\n";
      print PLOT "set label 2 \"%\" at 0.55,103\n";
      print PLOT "set label 1 \"Coral/DAG NeTraMet, 5-min samples\\n" .
                  "Nevil Brownlee, U Auckland / CAIDA\" at 55000,20 right\n";
      print PLOT "plot \"$plotname.plt\"" .
               " u 2:3 t \"TCP out of SDSC [ToFlowPDUs]\" w st 5" . # magenta
         ", \"\" u 2:4 t \"TCP into SDSC [FromFlowPDUs]\" w st 1" . # red
         ", \"\" u 2:5 t \"UDP out of SDSC [ToFlowPDUs]\" w st 4" . # cyan
         ", \"\" u 2:6 t \"UDP into SDSC [FromFlowPDUs]\" w st 3" . # blue
         "\n";
      close (PLOT);
      }
   elsif ($which == 18) {
      $plotname = "$iprefix.flowoctets.cdst";
      open_tmp(\*PLOT, $tempname = "$ruleset.$plotname.tmp");
      print PLOT "set title \"Cumulative Stream Size distribution observed at SDSC, " .
         "00.$iprefix\"\n";
      print PLOT "set xlabel \"Stream Size, kiloBytes\" +25, +0\n";
      print PLOT "set logscale x\n";
      print PLOT "set xrange[0.08:6500]\n";
      print PLOT "set yrange[0:100]\n";
      print PLOT "set ytics 0,10\n";
      print PLOT "set label 2 \"%\" at 0.055,104\n";
      print PLOT "set label 1 \"Coral/DAG NeTraMet, 5-min samples\\n" .
                  "Nevil Brownlee, U Auckland / CAIDA\" at 5000,20 right\n";
      print PLOT "plot \"$plotname.plt\"" .
               " u 2:3 t \"TCP out of SDSC [ToFlowOctets]\" w st 5" . # magenta
         ", \"\" u 2:4 t \"TCP into SDSC [FromFlowOctets]\" w st 1" . # red
         ", \"\" u 2:5 t \"UDP out of SDSC [ToFlowOctets]\" w st 4" . # cyan
         ", \"\" u 2:6 t \"UDP into SDSC [FromFlowOctets]\" w st 3" . # blue
         "\n";
      close (PLOT);
      }

   else { return; }

   if ($eps_plot) {
      printf ">>> gnuplot $tempname > $out_prefix.$plotname.eps\n";
      system("gnuplot $tempname > $out_prefix.$plotname.eps");
      }
   else {
      printf ">>> gnuplot $tempname > $out_prefix.$plotname.png\n";
      system("gnuplot $tempname > $out_prefix.$plotname.png");
      }
   #   unlink "$tempname";
   }

sub get_dist_params {
   my ($from_file, $st_ref,$dist_ref,$cols_ref) = @_;  # Arguments

   if (defined($from_file)) {
      open(FROM, $from_file)
      or die "Can't open from_file: $!\n";
      }
 
   my (@attribs, @lo, @hi, @nvalues);
   my $n = 0;  my $mycols = 1;

   while (<FROM>) { # read  flows file
      chop;  # Trim new line
      if (/^#/) {  # Starts with a #
         if (/^##NeTraMet /) {
            if (/\(UTC\)/) {  $utc_in = 1; }
	     print " ==== utc_in = 1\n";
            }
         elsif (/^##plt_stat /) {
            my @items = split(/ +/, $_);
            foreach my $item (@items) {
               if ($item =~ /(.+)=(\d+)\.\.(\d+)/) {
                  $attribs[$n] = $1;
	          $lo[$n] = $2;  $hi[$n] = $3;
                  $nvalues[$n] = $3-$2+1;
	          $mycols *= $nvalues[$n];
	          ++$n;
                  }
               elsif ($item =~ /stats=(\d+)/) {
                  $$st_ref = $1;
                  }
               elsif ($item =~ /distribs=(\d+)/) {
                  $$dist_ref = $1;  $mycols *= $ndistribs;
                  }
               elsif ($item =~ /cols=(\d+)/) {
                  $$cols_ref = $1;
                  }
               }
            close(FROM);
            return;
	    }
	 } 
      next;
      }
   }

sub put_multiplot {
   my ($FILE, $dname, $site, $tz, $title,
      $ncols, $nkinds, $plot_type,
      $key1, $key2, $key3, $key4,
      $fc1,$kind1, $fc2,$kind2, $fc3,$kind3, $fc4,$kind4) = @_;

   put_gnu_header($FILE, 't', $xstart,$xfinish);

   my ($ylo,$yhi, $yunits, $units_x,$units_y, $plotname_y,  # Plot layouts
      $pl_org_x, $pl_scale_x,
      $cc_org_x,$cc_org_y, $cc_scale_x,$cc_scale_y);

   if ($plot_type == 1) {
      $ylo = 8;  $yhi = 80000;  $plotname_y = 25000;
      $yunits = 'ms';  $units_x = $ylab_x;  $units_y = 60000;
      $pl_org_x = 0;  $pl_scale_x = 1; 
      $cc_org_x = 0.0248;  $cc_org_y = 0.033;
      $cc_scale_x = 0.95;  $cc_scale_y = 0.22;
      }
   elsif ($plot_type == 2) {
      $ylo = 0.08;  $yhi = 500;  $plotname_y = 150;
      $yunits = 'kB';  $units_x = $ylab_x;  $units_y = 450;
      $pl_org_x = 0.0247;  $pl_scale_x = 0.968; 
      $cc_org_x = 0.0248;  $cc_org_y = 0.033;
      $cc_scale_x = 0.95;  $cc_scale_y = 0.22;
      }

   get_dist_params($data_in, \$nstats,\$ndistribs,\$ncols);
   my $n_servers = $ncols/2;
   print "nstats=$nstats, ndistribs=$ndistribs, ncols=$ncols, n_servers=$n_servers\n";
   $time_label = $utc_in ? 'Time: ddThh (UTC)' : "Local Time$tz";

   my ($p50, $p75, $p95, $p100,
      $cc1, $cc2, $cc3, $cc4);

   print $FILE "set multiplot\n";
   print $FILE "set key top right\n";

   print $FILE "set origin $cc_org_x, $cc_org_y\n";
   print $FILE "set size $cc_scale_x, $cc_scale_y\n";
   print $FILE "set yrange[0:139000]\n";
   print $FILE "set ytics (\"25\" 25000, \"50\" 50000," .
      " \"75\" 75000, \"100\" 100000, \"125\" 125000)\n";

   print $FILE "set label 2 \"kilostreams per sample\" at $logo_x, 115000\n";
   print $FILE "set xlabel \"$time_label\" +32, +0\n";
   if ($logo_reqd) {
      print $FILE "set label 3 \"Coral NeTraMet, 5-min samples\\n" .
                  "Nevil Brownlee, U Auckland / CAIDA\" at $logo_x, -60000\n";
    }
   print $FILE "set key right\n";

   $cc1 = $fc1 + 5*$ncols;  $cc2 = $fc2 + 5*$ncols;
   $cc3 = $fc3 + 5*$ncols;  $cc4 = $fc4 + 5*$ncols;

   print $FILE "plot \"$dname\" u 1:$cc4 t \"+$kind4\" w li $l1_c," .
                  " \"$dname\" u 1:$cc3 t \"+$kind3\" w li $d1_c," .
                  " \"$dname\" u 1:$cc2 t \"+$kind2\" w li $l2_c," .
                  " \"$dname\" u 1:$cc1 t \"$kind1\" w li $d2_c";
   print $FILE "\n";

   print $FILE "set logscale y\n";
   print $FILE "set yrange [$ylo:$yhi]\n";

   print PLOT "set ytics (";  # Labels at powers of 10, tics at 2,5
   my $f_mk = 0.01;
   for (; $f_mk < $ylo; $f_mk *= 10) {}
   my $intro = '';
   for (my $y = $f_mk; $y < $yhi; $y *= 10) {
#      my $str = sprintf "%s\"$y\" %.1f, \"\" %.1f, \"\" %.1f",
#         $intro, $y, $y*2, $y*5;
      my $str = sprintf "%s\"$y\" %.1f, \"\" %.1f",
         $intro, $y, $y*3;
      printf $FILE $str;
      $intro = ',';
      }
   print PLOT ")\n";

   print $FILE "set title \"$title\"\n";
   print $FILE "set label 1 \"$yunits\" at $units_x,$units_y right\n";
   print $FILE "set xlabel\n";
   if ($logo_reqd) {
      print $FILE "set nolabel 3\n";
      }
   print $FILE "set origin $pl_org_x, 0.76\n";
   print $FILE "set size $pl_scale_x, 0.23\n";
   $p50 = $fc1;
   $p75 = $fc1 + $ncols;  $p95 = $fc1 + 3*$ncols;  $p100 = $fc1 + 2*$ncols;

   print $FILE "set label 2 \"$kind1\" at $logo_x, $plotname_y\n";
   print $FILE "plot \"$dname\" u 1:$p95 t \"$key2\" w li $l2_c," .
      " \"$dname\" u 1:$p75 t \"$key3\" w li $d1_c," .
      " \"$dname\" u 1:$p50 t \"$key4\" w li $l1_c\n";
   print $FILE "set title\n";

   print $FILE "set origin $pl_org_x, 0.59\n";
   print $FILE "set size $pl_scale_x, 0.192\n";
   $p50 = $fc2;
   $p75 = $fc2 + $ncols;  $p95 = $fc2 + 3*$ncols;  $p100 = $fc2 + 2*$ncols;

   print $FILE "set label 2 \"$kind2\" at $logo_x, $plotname_y\n";
   print $FILE "plot \"$dname\" u 1:$p95 t \"$key2\" w li $l2_c," .
      " \"$dname\" u 1:$p75 t \"$key3\" w li $d1_c," .
      " \"$dname\" u 1:$p50 t \"$key4\" w li $l1_c\n";

   print $FILE "set origin $pl_org_x, 0.415\n";
   print $FILE "set size $pl_scale_x, 0.192\n";
   $p50 = $fc3;
   $p75 = $fc3 + $ncols;  $p95 = $fc3 + 3*$ncols;  $p100 = $fc3 + 2*$ncols;

   print $FILE "set label 2 \"$kind3\" at $logo_x, $plotname_y\n";
   print $FILE "plot \"$dname\" u 1:$p95 t \"$key2\" w li $l2_c," .
      " \"$dname\" u 1:$p75 t \"$key3\" w li $d1_c," .
      " \"$dname\" u 1:$p50 t \"$key4\" w li $l1_c\n";

   print $FILE "set origin $pl_org_x, 0.24\n";
   print $FILE "set size $pl_scale_x, 0.192\n";
   $p50 = $fc4;
   $p75 = $fc4 + $ncols;  $p95 = $fc4 + 3*$ncols;  $p100 = $fc4 + 2*$ncols;

   print $FILE "set label 2 \"$kind4\" at $logo_x, $plotname_y\n";
   print $FILE "plot \"$dname\" u 1:$p95 t \"$key2\" w li $l2_c," .
      " \"$dname\" u 1:$p75 t \"$key3\" w li $d1_c," .
      " \"$dname\" u 1:$p50 t \"$key4\" w li $l1_c\n";

   print $FILE "set nomultiplot\n";
   }

sub put_distrib_plot {
   my ($FILE, $site, $tz, $title,
      $iprefix, $plotname, $plot_type,
      $fc1,$kind1, $fc2,$kind2, $fc3,$kind3, $fc4,$kind4) = @_;

   my ($xlo,$xhi,$xlabel,  $ylo,$yhi, $logaxes, $keypos,
      $obsx,$obsy, $logox, $logoy);  # Plot layouts
   if ($plot_type == 1) {  # Counts vs lifetimes
      $xlo = 8;  $xhi = 200000;  $ylo = 8;  $yhi = 60000;
      $xlabel = 'Stream lifetime, ms';
      $logaxes = 'xy';  $keypos = 'top right';
      $obsx = 20;  $obsy = 35000;
      $logox = 15;  $logoy = 4.3;
      }
   elsif ($plot_type == 2) {  #  Percent vs lifetimes
      $xlo = 9;  $xhi = 200000;  $ylo = 0;  $yhi = 100;
      $xlabel = 'Stream lifetime, ms';
      $logaxes = 'x';  $keypos = 'bottom right';
      $obsx = 13;  $obsy = 5;
      $logox = 15;  $logoy = -7;
      }
   elsif ($plot_type == 3) {  #  Percent vs stream sizes
      $xlo = 0.09;  $xhi = 100;  $ylo = 0;  $yhi = 100;
      $xlabel = 'Stream Size, kB';
      $logaxes = 'x';  $keypos = 'bottom right';
      $obsx = 0.105;  $obsy = 5;
      $logox = 15;  $logoy = -7;
      }

   put_gnu_header(\*PLOT, '');
   print $FILE "set multiplot\n";
   print $FILE "set origin 0,0.05\n";
   print $FILE "set size 1,0.95\n";
   print $FILE "set title \"$title\"\n";
#   print $FILE "set title \"Stream lifetime distribution [FlowTime] at UCSD\"\n";
   print $FILE "set xlabel \"$xlabel\" +28, +0\n";
   print $FILE "set logscale $logaxes\n";
   print $FILE "set xrange[$xlo:$xhi]\n";
   print $FILE "set yrange[$ylo:$yhi]\n";
   print $FILE "set key $keypos\n";

   print $FILE "set label 1\"Observed at 2001$iprefix\" at $obsx,$obsy\n";
   if ($logo_reqd) {
      print $FILE "set label 2\"Coral/DAG NeTraMet, 5-min samples\\n" .
               "Nevil Brownlee, U Auckland / CAIDA\" at $logox,$logoy\n";
      }
   print PLOT
      "plot \"$plotname\" u 2:$fc1 t \"$kind1\" w steps $d2_c" . # red
      ", \"\" u 2:$fc2 t \"$kind2\" w steps $l2_c" . # magenta
      ", \"\" u 2:$fc3 t \"$kind3\" w steps $d1_c" . # blue
      ", \"\" u 2:$fc4 t \"$kind4\" w steps $l1_c" . # green
      "\n";
   print $FILE "set nomultiplot\n";
   }


sub put_gnu_header {
   my ($FILE, $timefmt) = @_;  # Arguments

   if ($eps_plot) {
      print $FILE "set term postscript eps monochrome\n";
      }
   else {
      print $FILE "set term png small color\n";
      }

#  print $FILE "set term gif small size 640,480 xffffff x000000 x404040 \\\n",
#                 "xff0080 xff6600 x00ff40 x0080ff\n";  # red -> blue
# last three colours used for 10, 3, 1 contours
# if gnuplot finds four contours, it uses four colours, so we'd need
#    to specify one more!
#                 "xff0080 xff0000 xff6600 xfca503 \\\n",
#                 "xffffd8 x00ff40 x00ffff x0080ff\n";  # red -> blue

   print $FILE "set missing \"?\"\n";
   if ($timefmt ne '') {
      print $FILE "set timefmt \"%m%d.%H%M\"\n";
      print $FILE "set xdata time\n";
      print $FILE "set format x \"%02dT%H\"\n";  # ISO 8601 uses T (Ken Keys)
      if (defined($xstart)) {
         print PLOT "set xrange [$xstart:$xfinish]\n";
         }
      }
   }

sub open_tmp {
    my ($FILE, $tempname) = @_;  # Arguments
   if (!open($FILE, ">$tempname") ) {
      printf STDERR "Couldn't open $tempname for write <<<\n";
      exit;
      }
   }


sub u_seconds {
   my ($t_day) = @_;  # Arguments

   my ($fy, $fm, $fd);
   $t_day =~ /^(\d\d\d\d)(\d\d)(\d\d)/;
   $fy = $1;  $fm = $2;  $fd = $3;
   return timelocal(0, 0, 0, $fd, $fm-1, $fy-1900);
   }

sub modify_date {
   my ($t_day, $d_incr) = @_;  # Arguments

   my $fs = u_seconds($t_day);

   my @fyd = localtime($fs + $d_incr * 24*60*60);  # + hours
   my $r = sprintf "%04d%02d%02d",
      $fyd[5]+1900, $fyd[4]+1, $fyd[3];  # yyyy mm dd
   return $r;
   }

sub g_fmt_date {
   my ($t_day, $h_incr) = @_;  # Arguments

   my $fs = u_seconds($t_day);

   if (defined($h_incr)) { $fs += $h_incr*60; }

   my @fyd = localtime($fs);
   my $r = sprintf '"%02d%02d.%02d%02d"',
      $fyd[4]+1, $fyd[3], $fyd[2], $fyd[1];  # mm dd hh mm
   return $r;
   }
